import scipy as sc
import scipy.sparse as sp
import numpy as np
import pandas as pd
import os
import re
import matplotlib.pyplot as plt

def timereport(path, file):
    data = pd.read_csv(path + "report/" + file)
    data = data.to_numpy()
    fig = plt.figure()
    plt.plot(data)
    plt.xlabel("Generation")
    plt.ylabel("Compute time")
    plt.show()  

def basic_plotter(num, c, mutlim, path, select=0):
    # path = "E:/Simulations/Pos Neg Evolution/x5 Test/Test1/%5Ctest"
    
    y = [0 for i in range(num)]
    x = [i for i in range(num)]
    
    for i in range(num):
        a = sc.sparse.load_npz(path + "_" + str(i) + ".npz")
        y[i] = a._shape[0]
        if i % c == 0:
            temp = []
            if select:
                temp= a[:,1].toarray() + a[:,2].toarray()
            else:
                temp = a[:,1].toarray()
            fig = plt.figure()
            plt.hist(temp)
            plt.xlim(mutlim)
            plt.xlabel("Mutation number")
            plt.ylabel("Cells number")
            plt.show()
            
            # temp = a[:,0].toarray()
            # fig = plt.figure()
            # plt.hist(np.log10(temp))
            # plt.xlim(mutlim)
            # plt.xlabel("Fitness parameter [log10]")
            # plt.ylabel("Cells number")
            # plt.show()
    
    fig = plt.figure()
    plt.plot(x,y)
    plt.xlabel("Generation")
    plt.ylabel("Population size")
    plt.show()   
  
def npztotxt(path_in, fname):
    a = sc.sparse.load_npz(path_in + fname)
    df = pd.DataFrame(a.toarray())
    if a._shape[1] == 3:
        df.columns = ["fitness", "positive mutation number", "negative mutation number"]
        os.makedirs(path_in + "/CSV/", exist_ok=True)
        df.to_csv(path_in + "/CSV/" + fname.rstrip(".npz") + ".csv", index=False)
    else:
        columns = ["fitness", "mutation number"]
        # for i in range(a._shape[0] - 1):
        #     columns.append(str(i))
        df.columns = columns      
        os.makedirs(path_in + "/CSV/", exist_ok=True)
        df.to_csv(path_in + "/CSV/" + fname.rstrip(".npz") + ".csv", index=False)
 
def stoalfafit(path, mi, ma, step):
    dt = []
    _filter = (lambda x: [a.endswith('.npz') for a in x])
    columns = []
    for i in range(1,26,1):
        columns = np.append(columns, np.repeat("%.4f" % ((i-1)*step + mi), 3))
        files = np.array(os.listdir(path + str(i)))
        files = files[_filter(files)]
        a = np.zeros((len(files), 3))
        for j in files:
            idx = int(float(re.search(r'\d+.\d+', j[::1]).group())/25)
            file = sp.load_npz(path + str(i) + '/' + j).toarray()
            a[idx,0] = np.sum(file, axis=0)[1] / len(file)
            a[idx,1] = np.std(file[:,1])
            a[idx,2] = np.median(file[:,1])
        if dt == []:
            dt = a
        else:
            dt = np.append(dt, a, axis=1)
            
    
    df = pd.DataFrame(dt)
    df.columns = columns
    df.to_csv(path + 'data.csv')
 
if __name__ == "__main__":
    path = 'E:/Simulations/Stationary Wave/stationary_1d_for_s/'
    stoalfafit(path, 0.0001, 0.0049, 0.0002)
    # t = [float(x) for x in range(0,5351,25)]
    # fname = "pos_n_normal"
    # # file = "pos_report_3.txt"
    # path = "E:/Simulations/Stationary Wave/Normal positive stationary/"
    # # # timereport(path, file)
    # # basic_plotter(t, 10, (0,2000), path + fname)
    # # # npztotxt("E:/Simulations/Pos Neg Evolution/x5 Test/Test1/", "%5Ctest_1349.npz")
    # for i in t:
    #     name = fname + "_" + str(i) + ".npz"
    #     npztotxt(path, name)
    # alfa = 10
    # x = np.array([x/100 for x in range(100000)])
    # a = x > 1
    # b = 1 - np.exp(-alfa * x)
    # c = np.arctan(alfa*x) / (np.pi/2)
    # d = -1 / (alfa*x + 1) + 1
    # plt.plot(x, a)
    # plt.plot(x, b)
    # plt.plot(x, c)
    # plt.plot(x, d)
    


import PyQt5.QtWidgets as qtWidget
import PyQt5.QtGui as qtGui
from PyQt5.QtCore import Qt
import sys
import os
import numpy as np
import scipy as sc
import pandas as pd
from pathlib import Path
import copy
import re

from threading import Thread
from multiprocessing import Process, Queue

from seEvo_evolution_init import seEvoInit
import externalPlots as externalPlots
# from seEvo1D.seEvo_evolution_init import seEvoInit
# import seEvo1D.externalPlots as externalPlots

disclaimer = '''
    seEvo - slightly effect evolution simulator basing on Gillespie algorithm.
    Copyright (C) 2023 by Jarosław Gil. 
    This program comes with ABSOLUTELY NO WARRANTY.
    This is free software, and you are welcome to redistribute it
    under certain conditions.
    \n
    Software can provide multiple simulation - start simulation button starts new process.
    \n
    Example usage:
        1. Set file name and path on general tab than select save files on output tab.
        Population matrix will be saved to '.npz' files interpreted by software with 'Skip generation save' freqency
        2. Set parameters - default values already are inserted
        3. Start simulation. On status field will appear simulation ID and updated generation number.
        4. Simulation end will be annotated. To finish simulation earlier type simulation id and push stop simulation button.
        5. After simulation push wanted output figure and select file from which it should be processed
    '''

output_about = '''
    single mutation wave plot - select one or multiple files to plot mutation wave (mutation number vs cells number)
    population growth plot - select multiple files to plot how population grow between generations
    multiple mutation wave plot - select two or more files to combine mutation wave plots on one plot (able for normal and analytical wave plot)
    evolution dynamics - comparision of analytical and normal model data - select whole population for both versions (generations should be corresponding).
        plot will contain three mutation waves in 1/4, 2/4, 3/4 of generation size, population groth and mutation changes throught evolution
'''

class mainFormat(qtWidget.QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.q = Queue()
        
        self.ID = 1
        self.idx_s = 0
        self.th_s = []
        self.s_ID = []
        
        self._th_pg = Process()
        self._th_cp = Process()
        self._th_mw = Process()
        self._th_fw = Process()
        self._th_ed = Process()
        self.createMainView()
        self._monitor = True
        self.th_monitor = Thread(target=self.monitor, args=(self.q, self.status))
        self.th_monitor.start()
    
    def closeEvent(self, event):
        for i in self.s_ID:
            self.q.put(['1', str(i.split(',')[0]), "exit"])
            
        for i in self.th_s:
            i.join()
            
        self._monitor = False
        self.th_monitor.join()
        event.accept()   
    
    def monitor(self, q, status):
        while self._monitor:
            if not q.empty():
                data = q.get()
                if data[0] == '0':
                    status.setText("ID: " + data[1] + ", generation: " + data[2])
                elif data[0] == 'exit':
                    status.setText("stopped: " + data[1])
                    for i in self.s_ID:
                        if i == data[1]:
                            idx = self.s_ID.index(i)
                            self.s_ID.remove(self.s_ID[idx])
                            self.th_s[idx].join()
                            self.th_s.remove(self.th_s[idx])
                            self.idx_s = self.idx_s - 1
                else:
                    q.put(data)
                    
            
            if self._th_mw.is_alive():
                self._th_mw.join(1)   
                if not self._th_mw.is_alive():                 
                    self.status.setText("mutation wave done")
                    # self._th_mw = None
            if self._th_fw.is_alive():
                self._th_fw.join(1) 
                if not self._th_fw.is_alive():                   
                    self.status.setText("fitness wave done")
                    # self._th_fw = None
            if self._th_pg.is_alive():
                self._th_pg.join(1) 
                if not self._th_pg.is_alive():                   
                    self.status.setText("population growth done")
                    # self._th_fw = None
            if self._th_cp.is_alive():
                self._th_cp.join(1) 
                if not self._th_cp.is_alive():                   
                    self.status.setText("combined plot done")
                    # self._th_fw = None
            if self._th_ed.is_alive():
                self._th_ed.join(1)   
                if not self._th_ed.is_alive():                 
                    self.status.setText("evolution dynamics done")
                    # self._th_mw = None
    
    def update(self):
        i = self.tabs.indexOf(self._tabUI)
        self.tabs.removeTab(i)
        self._tabUI = self.threadTabUI()
        self.tabs.addTab(self._tabUI, "Active threads")
    
    def about(self):
        global disclaimer
        self.showDialog(disclaimer, 'About')
            
    def fitWaveAction(self):
        if self._th_fw.is_alive():
            self.showDialog("plottin already running", 'Info')
            return
        fname = qtWidget.QFileDialog.getOpenFileNames(None, 'Open file', "Z://","Data File (*.npz)")[0] 
        if len(fname) == 0:
            self.showDialog("No file selected!", "Alert")
            return
        if all(map(lambda x: x.endswith('.npz'), fname)):
            self._th_fw = (Process(target=externalPlots.fitnessWave, args=(fname,)))
            # externalPlots.fitnessWave(fname)
            self._th_fw.start()
            self.status.setText("fitness wave plot ongoing")
        else:
            self.showDialog("Wrong file/files extension. Use only one kind at time", "Alert")
            
    def mutWaveAction(self):
        if self._th_mw.is_alive():
            self.showDialog("plottin already running", 'Info')
            return
        fname = qtWidget.QFileDialog.getOpenFileNames(None, 'Open file', "Z://","Data File (*.npz)")[0] 
        if len(fname) == 0:
            self.showDialog("No file selected!", "Alert")
            return
        if all(map(lambda x: x.endswith('.npz'), fname)):
            self._th_mw = (Process(target=externalPlots.mutationWave, args=(fname,)))
            # externalPlots.mutationWave(fname)
            self._th_mw.start()
            self.status.setText("mutation wave plot ongoing")
        else:
            self.showDialog("Wrong file/files extension. Use only one kind at time", "Alert")
            
    def popGrowthAction(self):
        if self._th_pg.is_alive():
            self.showDialog("plottin already running", 'Info')
            return
        fname = qtWidget.QFileDialog.getOpenFileNames(None, 'Open file', "Z://","Data File (*.npz)")[0] 
        if len(fname) == 0:
            self.showDialog("No file selected!", "Alert")
            return
        if all(map(lambda x: x.endswith('.npz'), fname)):
            self._th_pg = (Process(target=externalPlots.popGrowth, args=(fname,)))
            # externalPlots.popGrowth(fname)
            self._th_pg.start()
            self.status.setText("population growth plot ongoing")
        else:
            self.showDialog("Wrong file/files extension. Use only one kind at time", "Alert")
            
    def combinedMutWave(self):
        if self._th_cp.is_alive():
            self.showDialog("plottin already running", 'Info')
            return
        fname = qtWidget.QFileDialog.getOpenFileNames(None, 'Open file', "Z://","Data File (*.npz)")[0] 
        if len(fname) == 0:
            self.showDialog("No file selected!", "Alert")
            return
        if all(map(lambda x: x.endswith('.npz'), fname)):
            self._th_cp = (Process(target=externalPlots.combainedMutWave, args=(fname,)))
            # externalPlots.combainedMutWave(fname)
            self._th_cp.start()
            self.status.setText("combined mutation wave plot ongoing")
        else:
            self.showDialog("Wrong file/files extension. Use only one kind at time", "Alert")
            
    def evolutionDynamics(self):
        if self._th_ed.is_alive():
            self.showDialog("plottin already running", 'Info')
            return
        # self.showDialog("First select ANALYTICAL .npz folder (whole population)\n Second select NORMAL .npz folder (whole population)", "INFO")
        # fname_an = qtWidget.QFileDialog.getOpenFileName(None, 'Select Analytical Files', "Z://","Data File (*.npz)")[0] 
        fname = qtWidget.QFileDialog.getExistingDirectory(self,"Choose Directory","Z:/")
        if fname == '':
            self.showDialog("No analytical folder selected", "Alert")
            return
        fname_an = np.array(os.listdir(fname))
        fname_an = np.array(list(map(lambda x,y: (y + '/' + x)*x.endswith('.npz'), fname_an, np.repeat(fname, len(fname_an)))))
        fname_an = fname_an[fname_an != '']
        if not all(map(lambda x: 'analytical' in x, fname_an)):
            self.showDialog("Select only analytical files", "Alert")
            return
        # fname_norm = qtWidget.QFileDialog.getOpenFileNames(None, 'Select Normal Files', "Z://","Data File (*.npz)")[0] 
        fname = qtWidget.QFileDialog.getExistingDirectory(self,"Choose Directory","Z:/")
        if fname == '':
            self.showDialog("No normal folder selected", "Alert")
            return
        fname_norm = np.array(os.listdir(fname))
        fname_norm = np.array(list(map(lambda x,y: (y + '/' + x)*x.endswith('.npz'), fname_norm, np.repeat(fname, len(fname_norm)))))
        fname_norm = fname_norm[fname_norm != '']
        if not all(map(lambda x: 'normal' in x, fname_norm)):
            self.showDialog("Select only normal files", "Alert")
            return
        if all(map(lambda x: x.endswith('.npz'), fname_an)) and all(map(lambda x: x.endswith('.npz'), fname_norm)):
            self._th_ed = (Process(target=externalPlots.evolutionDynamics, args=(fname_an, fname_norm)))
            # externalPlots.evolutionDynamics(fname_an, fname_norm)
            self._th_ed.start()
            self.status.setText("evolution dynamics plot ")
        else:
            self.showDialog("Wrong file/files extension.", "Alert")
    
    def createMainView(self):
        layout = qtWidget.QGridLayout()
        self.tabs = qtWidget.QTabWidget()
        self.tabs.addTab(self.generalTabUI(), "General")
        self.tabs.addTab(self.parametersTabUI(), "Parameters")          
        self.tabs.addTab(self.outputTabUI(), "Output")   
        self._tabUI = self.threadTabUI()
        self.tabs.addTab(self._tabUI, "Threads")     
        row = 1
        layout.addWidget(self.tabs, row, 0, 1, 5)
        
        start = qtWidget.QPushButton(self)
        start.setText("Start simulation")
        start.clicked.connect(self.simStart)
        row = row + 1
        layout.addWidget(start, row, 0, 1, 5)
        
        stop = qtWidget.QPushButton(self)
        stop.setText("Stop simulation")
        stop.clicked.connect(self.simStop)
        row = row + 1
        self._ID = qtWidget.QLineEdit(str(0))        
        self._ID.setValidator(qtGui.QIntValidator(0, 1000))
        layout.addWidget(self._ID, row, 0, 1, 2)
        layout.addWidget(stop, row, 2, 1, 3)
        
        self.status = qtWidget.QLabel()
        self.status.setText("Stopped")
        self.status.setAlignment(Qt.AlignCenter)
        self.status.setFont(qtGui.QFont('Consolas', 15))
        row = row + 1  
        layout.addWidget(self.status, row, 0, 1, 5)
        
        about = qtWidget.QPushButton(self)
        about.setText("About")
        about.clicked.connect(self.about)
        row = row + 1
        layout.addWidget(about, row, 0, 1, 5)
        
        self.setLayout(layout)
        
    def generalTabUI(self):
        generalTabUI = qtWidget.QWidget()
        layout = qtWidget.QGridLayout()

        self._NONE = qtWidget.QRadioButton("Checked: Normal simulation")       
        self._analytical = qtWidget.QRadioButton("Checked: Analytical model")               
        self._NONE.click()
        
        self._file_name = qtWidget.QLineEdit()
        _file_name_label = qtWidget.QLabel()
        _file_name_label.setText('File name')
        self._file_path = qtWidget.QLineEdit()
        _file_path_label = qtWidget.QLabel()
        _file_path_label.setText('File path')
        _file_path_button = qtWidget.QPushButton(self)
        _file_path_button.setText('Select path')
        _file_path_button.clicked.connect(self.selectPath)
        
        row = 0
        layout.addWidget(_file_name_label, row, 0)
        layout.addWidget(self._file_name, row, 1, 1, 2)
        row = row + 1
        layout.addWidget(_file_path_label, row, 0)
        layout.addWidget(self._file_path, row, 1)
        layout.addWidget(_file_path_button, row, 2)
        
        _group = qtWidget.QButtonGroup() 
        _group.addButton(self._NONE)
        _group.addButton(self._analytical)
        row = row + 1
        layout.addWidget(self._NONE, row, 0)
        row = row + 1
        layout.addWidget(self._analytical, row , 0) 
        
        generalTabUI.setLayout(layout)
        return generalTabUI
        
    def parametersTabUI(self):
        parametersTab = qtWidget.QWidget()
        layout = qtWidget.QGridLayout()
        
        row = 0
        self._population = qtWidget.QLineEdit(str(10**4))
        layout.addWidget(qtWidget.QLabel("Initial population, N0"), row, 0)
        layout.addWidget(self._population, row, 1)
        layout.addWidget(qtWidget.QLabel("min: 0, max: 10^7"), row, 2)
        row = row + 1
        self._capacity = qtWidget.QLineEdit(str(10**4))
        layout.addWidget(qtWidget.QLabel("Enviroment capacity, Nc"), row, 0)
        layout.addWidget(self._capacity, row, 1)
        layout.addWidget(qtWidget.QLabel("min: 0, max: 10^7"), row, 2)
        row = row + 1
        self._steps = qtWidget.QLineEdit(str(10000))
        layout.addWidget(qtWidget.QLabel("Generations (simulation steps)"), row, 0)
        layout.addWidget(self._steps, row, 1)
        layout.addWidget(qtWidget.QLabel("number of steps,(optional: 'cells' as type of break - population size)"), row, 2)
        row = row + 1
        self._tau = qtWidget.QLineEdit(str(0.005))     
        layout.addWidget(qtWidget.QLabel("Tau step"), row, 0)
        layout.addWidget(self._tau, row, 1)
        layout.addWidget(qtWidget.QLabel("min: 0, max: 1, step: 0.001"), row, 2)
        row = row + 1
        self._skip = qtWidget.QLineEdit(str(25))
        layout.addWidget(qtWidget.QLabel("Skip generation save"), row, 0)
        layout.addWidget(self._skip, row, 1)
        layout.addWidget(qtWidget.QLabel("min: 1, max: 100, step: 1"), row, 2)
        row = row + 1
        self._mut_effect = qtWidget.QLineEdit(".0005")
        layout.addWidget(qtWidget.QLabel("Mutation effect, f/s"), row, 0)
        layout.addWidget(self._mut_effect, row, 1)
        layout.addWidget(qtWidget.QLabel("min: 0.00001, max: 0.99999"), row, 2)
        row = row + 1
        self._mut_prob = qtWidget.QLineEdit("0.025")  
        layout.addWidget(qtWidget.QLabel("Mutation probability, pf/ps"), row, 0)
        layout.addWidget(self._mut_prob, row, 1)
        layout.addWidget(qtWidget.QLabel("min: 0.00001, max: 0.99999"), row, 2)  
        row = row + 1
        self._mdt_exp = qtWidget.QLineEdit("1.0")  
        layout.addWidget(qtWidget.QLabel("Death intensity exponent, A"), row, 0)
        layout.addWidget(self._mdt_exp, row, 1)
        
        _save_params = qtWidget.QPushButton(self)
        _save_params.setText('Save Parameters')
        _save_params.clicked.connect(self.saveParams)
        row = row + 1
        layout.addWidget(_save_params, row, 0, 1, 3)
        
        _load_params = qtWidget.QPushButton(self)
        _load_params.setText('Load Parameters')
        _load_params.clicked.connect(self.loadParams)
        row = row + 1
        layout.addWidget(_load_params, row, 0, 1, 3)
        
        parametersTab.setLayout(layout)
        return parametersTab
    
    def saveParams(self):
        params, break_type = self.validateParams()
        
        try:
            if self._file_name.text() == "":
                raise Exception()
            if self._file_path.text() == "":
                raise Exception()
        except:
            self.showDialog("Enter save localization and name", "Alert")
            return
            
        dfp = pd.DataFrame(params)
        
        filepath = Path(self._file_path.text() + '/params'  + ".csv")  
        filepath.parent.mkdir(parents=True, exist_ok=True)  
        dfp.to_csv(filepath)  
    
    def loadParams(self):
        fname = qtWidget.QFileDialog.getOpenFileName(self, 'Open file', "Z://","CSV files (*.csv)") 
        if fname[0] == '':
            self.showDialog("No file selected!", "Alert")
            return
        df = pd.read_csv(fname[0])
        df = df.to_numpy()        
    
        self._population.setText(str(int(df[0,1])))
        self._capacity.setText(str(int(df[1,1])))
        self._steps.setText(str(int(df[2,1])))
        self._tau.setText(str(df[3,1]))
        self._skip.setText(str(df[4,1]))
        self._mut_prob.setText(str(df[5,1]))
        self._mut_effect.setText(str(df[6,1]))        
        self._mdt_exp.setText(str(df[7,1]))
    
    def outputTabUI(self):
        outputTabUI = qtWidget.QWidget()
        layout = qtWidget.QGridLayout()        
        
        self._sx = qtWidget.QCheckBox("Save files")        
        row = 0
        layout.addWidget(self._sx, row, 0)

        _mut_wave = qtWidget.QPushButton(self)
        _mut_wave.setText('Mutation wave plot - single mutation wave on figure')
        _mut_wave.clicked.connect(self.mutWaveAction)
        row = row + 1
        layout.addWidget(_mut_wave, row, 0)
        
        # _fit_wave = qtWidget.QPushButton(self)
        # _fit_wave.setText('Fitness wave plot')
        # _fit_wave.clicked.connect(self.fitWaveAction)
        # row = row + 1
        # layout.addWidget(_fit_wave, row, 0)
        
        _pop_growth = qtWidget.QPushButton(self)
        _pop_growth.setText('Population growth')
        _pop_growth.clicked.connect(self.popGrowthAction)
        row = row + 1
        layout.addWidget(_pop_growth, row, 0)
        
        _combined = qtWidget.QPushButton(self)
        _combined.setText('Mutation wave plot - multiple mutation waves on figure')
        _combined.clicked.connect(self.combinedMutWave)
        row = row + 1
        layout.addWidget(_combined, row, 0)
        
        _dyn = qtWidget.QPushButton(self)
        _dyn.setText('Evolution dynamics')
        _dyn.clicked.connect(self.evolutionDynamics)
        row = row + 1
        layout.addWidget(_dyn, row, 0)
        
        _about = qtWidget.QPushButton(self)
        _about.setText('Short instruction')
        _about.clicked.connect(self.showAboutOutput)
        row = row + 1
        layout.addWidget(_about, row, 0)
        
        outputTabUI.setLayout(layout)
        return outputTabUI
    
    def showAboutOutput(self):
        self.showDialog(output_about, 'About')
        
    def showInactive(self):
        self.showDialog('Inactive', 'Info')
        
    def threadTabUI(self):
        threadTabUI = qtWidget.QWidget()
        layout = qtWidget.QFormLayout()
        
        _refresh = qtWidget.QPushButton("Refresh")
        _refresh.clicked.connect(self.update)
        
        layout.addRow("ID", _refresh)
        layout.addRow("Started", qtWidget.QLabel())
        
        for i in self.s_ID:
            layout.addRow(qtWidget.QLabel("ID: " + str(i)), qtWidget.QLabel())
        
        if self._th_pg.is_alive():
            layout.addRow(qtWidget.QLabel("population growth plot"), qtWidget.QLabel())
        if self._th_cp.is_alive():
            layout.addRow(qtWidget.QLabel("combined plot"), qtWidget.QLabel())
        if self._th_mw.is_alive():
            layout.addRow(qtWidget.QLabel("mutation wave"), qtWidget.QLabel())
        if self._th_fw.is_alive():
            layout.addRow(qtWidget.QLabel("fitness wave"), qtWidget.QLabel())
        if self._th_ed.is_alive():
            layout.addRow(qtWidget.QLabel("evolution dynamics"), qtWidget.QLabel())
        
        threadTabUI.setLayout(layout)
        return threadTabUI
    
    def simStart(self):
        params, break_type = self.validateParams()
        
        try:
            if self._file_name == "" and self._sx.isChecked():
                raise Exception()
            if self._file_path == "" and self._sx.isChecked():
                raise Exception()
        except:
            self.showDialog("Enter save localization and name", "Alert")
            return
        
        plots = 16*self._sx.isChecked()
        
        iPop = []
        if self._analytical.isChecked():
            iPop = np.array([[0],[float(params[0])]])
            select = 1
            name = 'analytical'
        else:
            iPop = np.ones(params[0])
            iMuts = np.zeros(params[0])
            iPop = sc.sparse.csr_matrix(np.array([iPop, iMuts]).T)
            # iPop = np.array([iPop, iMuts]).T
            select = 0
            name = 'normal'
          
        if self.ID == 1001:
            self.showDialog('To many simulations used. Please restart this app.', 'Memory alert')
            return 
        
        self.th_s.append(Process(target=seEvoInit, args=(copy.deepcopy(iPop), 
                                                    copy.deepcopy(params), 
                                                    self._file_name.text(), 
                                                    self._file_path.text(),
                                                    plots, self.q, self.ID, select, break_type))) 
        
        # seEvoInit(copy.deepcopy(iPop), 
        #     copy.deepcopy(params), 
        #     self._file_name.text(), 
        #     self._file_path.text(),
        #     plots, self.q, self.ID, select, break_type)
        
        self.s_ID.append(str(self.ID) + ', ' + name)
        self.ID = self.ID + 1
        self.th_s[self.idx_s].start()
        self.idx_s = self.idx_s + 1
        
    def simStop(self):
        msg = self._ID.text()
        try:
            msg = int(msg)
        except:
            self.showDialog('Type ID as integer in range (0,1000)', 'Error')
            return
        
        self.q.put(['1', str(msg), 'exit'])
    
    def validateParams(self):
        break_type = 0
        try:
            pop = int(self._population.text())
            cap = int(self._capacity.text())
            steps = self._steps.text().split(',')
            if len(steps) == 2:
                steps = int(steps[0])
                break_type = 1
            else:
                steps = int(steps[0])
            tau = float(self._tau.text())
            skip = float(self._skip.text())
            mut_prob = float(self._mut_prob.text())
            mut_effect = float(self._mut_effect.text())
            A = float(self._mdt_exp.text())
        except:
            self.showDialog("Type correct parameters","Alert")
            return
        return [pop, cap, steps, tau, skip, mut_prob, mut_effect, A], break_type
    
    def selectPath(self):
        dir_path = qtWidget.QFileDialog.getExistingDirectory(self,"Choose Directory","Z:/")
        self._file_path.setText(dir_path)

    def about(self):
        global disclaimer
        self.showDialog(disclaimer, 'About')

    def showDialog(self, text, title):
        msgBox = qtWidget.QMessageBox()
        msgBox.setIcon(qtWidget.QMessageBox.Information)
        msgBox.setText(text)
        msgBox.setWindowTitle(title)
        msgBox.setStandardButtons(qtWidget.QMessageBox.Ok)
         
        returnValue = msgBox.exec()

def run():
        app = qtWidget.QApplication(sys.argv)
        win = mainFormat()
        win.show()
        ret = app.exec_()
        sys.exit(ret)

if __name__ == "__main__":
    run()


import numpy as np
import time
import os
import math
import copy
import scipy as sc
import pandas as pd
from pathlib import Path  
from threading import Thread

from seEvo_evolution_loop_1D import seEvo1Dnorm
from seEvo_analytical_model_1D import seEvo1Danalytical
# from seEvo1D.seEvo_evolution_loop_1D import seEvo1Dnorm
# from seEvo1D.seEvo_analytical_model_1D import seEvo1Danalytical

end = False

def saveToFile(df, file_localization, file_name, iter_outer):    
    try:
        os.makedirs(file_localization, exist_ok=True) 
    except OSError as error:
        print(error)
    finally:
        sc.sparse.save_npz(file_localization + "/" + file_name + "_" + str(iter_outer), df)

def commands(q, ID, iPop, file_localization, file_name, iter_outer, skip, iter_inner, cycle, tau, select):
    global end
    queue_data = q.get()
    if(queue_data[0] == '1' and queue_data[1] == str(ID)):
        if(queue_data[2] == "exit"):
            print("exit")
            end = True
    else:
        q.put(queue_data)
 
    
def plotter(iPop, file_name, file_localization, iter_outer, plots, select):
    if plots & 1:
        print("TODO")
    if plots & 2:
        print("TODO")
    if plots & 16: 
        if select == 0:
            saveToFile(iPop, file_localization, file_name + '_normal', iter_outer)
        elif select == 1:
            saveToFile(sc.sparse.csr_matrix(iPop.T), file_localization, file_name + '_analytical', iter_outer)
        
    
def seEvoInit(iPop, 
              params, 
              file_name="", 
              file_localization="", 
              plots=0, q=None, ID=0, select=0, break_type=0):
    global end

    cap = params[1]
    steps = params[2]
    tau = params[3]
    skip = params[4]
    mut_effect = params[6]
    mut_prob = params[5]
    A = params[7]
    
    simTime = 0    
    iter_inner = 0
    iter_outer = 0
    cycle = round(skip/tau)
    
    t = time.time()
    tx = time.time()
    clear = False
    
    while 1:   
        if q != None:
            if not q.empty():
                commands(q, ID, iPop, file_localization, file_name, iter_outer, skip, iter_inner, cycle, tau, select)

        if iter_outer <= simTime:
            begin = 0
            t = time.time() - t  
            if select == 0:
                q.put(['0', str(ID), str(iter_outer)])
            elif select == 1 and iter_outer % 10 == 0:
                q.put(['0', str(ID), str(iter_outer)])
            
            if iter_inner * skip <= simTime:
                plotter(copy.deepcopy(iPop), file_name, file_localization, copy.copy(iter_inner * skip), plots, select)                
                iter_inner = iter_inner + 1
 
            clear = True
            
            if plots == 16:
                tx = time.time() - tx
                if not os.path.exists(file_localization + '/' + "report/"  + file_name + "_report_" + str(ID) + ".txt"):
                    os.makedirs(file_localization + '/' + "report/", exist_ok=True)
                    FILE = open(file_localization + '/' + "report/"  + file_name + "_report_" + str(ID) + ".txt", 'w')
                    FILE.write("name: %s" % file_name)
                    FILE.write('\n')
                    FILE.write(str(ID) + ',' + str(tx))
                    FILE.write('\n')
                    FILE.close()
                else:
                    FILE = open(file_localization + '/' + "report/"  + file_name + "_report_" + str(ID) + ".txt", 'a')
                    FILE.write(str(ID) + ',' + str(tx))
                    FILE.write('\n')
                    FILE.close()
                tx = time.time()            
            t = time.time()
            
            iter_outer = int(simTime) + 1
        
        if end:
            q.put(['exit', str(ID) + ', analytical' * (select == 1) + ', normal' * (select == 0)])
            break
        
        if not select:
            if (int(simTime) - steps >= 0 or iPop._shape[1] >= 10**6) and break_type == 0:
                print('all steps')
                end = True
            elif (iPop._shape[1] >= steps or iPop._shape[1] >= 10**6) and break_type == 1 and select != 2:
                print('all cells')
                end = True
        else:
            if (int(simTime) - steps >= 0 or sum(iPop[1,:]) >= 10**6) and break_type == 0:
                print('all steps')
                end = True
            elif (sum(iPop[1,:]) >= steps or sum(iPop[1,:]) >= 10**6) and break_type == 1 and select != 2:
                print('all cells')
                end = True
        
        if select == 0:            
            iPop, simTime = seEvo1Dnorm(iPop, cap, tau, A, mut_prob, mut_effect, simTime)
        elif select == 1:            
            iPop, simTime = seEvo1Danalytical(iPop, cap, tau, A, mut_prob, mut_effect, simTime)
            
        resume = 0
             


import numpy as np
import math
import itertools
import scipy as sc
import copy
from threading import Thread
from multiprocessing import Pool

mdv = [1]

def mf_dif(iPop, mdt, pdm, mut_effect):
    global mdv
    alfa = 0.12 #  0.0072*abs(mut_effect)**(-0.447) #
    Nfp = (1 - np.exp(-alfa * iPop))
    # Nfp = iPop > 1
    # Nfp = np.arctan(alfa * iPop) / (np.pi/2)
    # Nfp = -1 / (alfa * iPop + 1) + 1
    A = -mdt * iPop
    A = A + (1 - pdm) * mdv * iPop * Nfp
    A[1:len(iPop)] = A[1:len(iPop)] + pdm * mdv[0:len(mdv)-1] * iPop[0:len(iPop)-1] * Nfp[0:len(Nfp)-1]
    
    return A
    
def rk4(iPop, tau, mdt, pdm, mut_effect):
    k1 = tau*mf_dif(iPop[1,:], mdt, pdm, mut_effect)
    k2 = tau*mf_dif(iPop[1,:]+k1/2, mdt, pdm, mut_effect)
    k3 = tau*mf_dif(iPop[1,:]+k2/2, mdt, pdm, mut_effect)
    k4 = tau*mf_dif(iPop[1,:]+k3, mdt, pdm, mut_effect)
    iPop[1,:] = iPop[1,:]+(1/6)*(k1+2*k2+2*k3+k4)

def seEvo1Danalytical(iPop, cap, tau_x, A, mut_prob, mut_effect, simTime):
    global mdv
    popSize = sum(iPop[1,:])
    mdt = (popSize/cap)**A
    # tau = tau_x * cap/popSize
    tau = tau_x
    simTime = simTime + tau
    
    if iPop[1,len(iPop[0,:]) - 1] > 0:
        # mdv = np.append(mdv, math.exp(mut_effect * (mdv[len(mdv)-1] + 1)))
        c = np.array(range(int(iPop[0, len(iPop[0,:])-1])+1, int(iPop[0, len(iPop[0,:])-1])+501, 1))
        # mdv = np.append(mdv, np.exp(mut_effect * c))
        mdv = np.append(mdv, ((1+abs(mut_effect))**c)**(1-2*(mut_effect < 0)))
        a = len(iPop[0,:])
        iPop = np.array([np.append(iPop[0,:], np.zeros((500))), np.append(iPop[1,:], np.zeros((500)))])
        iPop[0,a:len(iPop[1,:])] = c
                
    rk4(iPop, tau, mdt, mut_prob, mut_effect)
    
    # if iPop[1,0] < 1:
    #     mdv = mdv[1:len(iPop[0,:])]
    #     iPop = iPop[:,1:len(iPop[0,:])]
    
    return iPop, simTime    